@@ -8,17 +8,18 @@ import uuid
 class AbstractComponent(ABC):
     build_target: str
     parent: any
+    absolute_root: any
     sub_components: list
 
+    cache_component: bool = False
+
+    used: bool = False
+
     @property
     @abstractmethod
     def code(self) -> str:
         ...
 
-    @abstractmethod
-    def grand_render(self):
-        ...
-
 
 class Component(AbstractComponent):
     def __init__(self, parent, add: bool = True, **kwargs):
@@ -30,14 +31,21 @@ class Component(AbstractComponent):
         for name, value in kwargs.items():
             setattr(self, name, value)
 
+        if self.cache_component and self.used:
+            add = False
+
         if not add:
             return
 
         if isinstance(self.parent, AbstractComponent):
+            self.absolute_root = self.parent.absolute_root
             self.parent.sub_components.append(self)
         else:
+            self.absolute_root = self.parent
             self.parent.builder.add_component(self)
 
+        self.used = True
+
     def __repr__(self):
         return f"{self.__class__.__name__}"
 
